{
    Respond, the fax number form daemon
    Copyright (C) 2000 Horst F <horstf@gmx.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}
unit History;

interface

uses
  Classes, SysUtils, Dialogs, Forms, Header, CommonDataAccess,
  ObservedStringList;

  type
    {
      This is a static history class
    }
    THistory = class(Tobject)
    private
      class procedure Initialize;
    public
      class procedure Add(const number, receiver: String);
      class function Count: integer;
      class function Get(const index: Integer;
          var number, receiver: String): Boolean;
    end;

implementation

  const
    initialized: boolean = false;
    histkey = 'History';

  var
    // The postions array, must be variant, because history size can be
    // set in options section
    cPositions: Variant;
    cCount: Integer;

  {
    Creates the positions array and initializes it with ascending integer
    values.
  }
  class procedure THistory.Initialize;
  var
    i: Integer;
    s: String;
  begin
    if not initialized then begin
      cCount := 0;
      try
        cPositions := VarArrayCreate([1, KMaxHistory], varInteger);
        for i := 1 to KMaxHistory do begin
          s := TCommonData.ReadString(HistorySection, histkey+IntToStr(i), '');
          if Length(s) = 0 then break
          else begin
            cPositions[i] := i;
            Inc(cCount);
          end;
        end;
        initialized := true;
      except
        on EVariantError do begin
          MessageDlg(GetLabelStr(KIErrOutOfMemory), mtError, [mbOK], 0);
          Application.Terminate;
        end;
      end;
    end;
  end;

  {
    Adds a number-receiver pair to the history.

    @param number phone number
    @param receiver receivers name
  }
  class procedure THistory.Add(const number, receiver: String);
  var
    i, j, k: Integer;
    sl: TObservedStringList;
  begin
    Initialize;
    sl := nil;
    try
      sl := TObservedStringList.Create;
      j := 0;
      // Search the history for the argument values
      for i := 1 to cCount do begin
        sl.Clear;
        TCommonData.ReadFields(HistorySection,
            histkey + IntToStr(cPositions[i]), sl);
        // Does this history entry match the new values?
        if (sl.Count > 0) and (sl.Strings[0] = number)
            and ((sl.Count = 1) and (Length(receiver) = 0)
            or (sl.Count > 1) and (sl.Strings[1] = receiver)) then begin
          // New entry found in history, now move it to the top
          j := cPositions[i];
          for k := i downto 2 do cPositions[k] := cPositions[k - 1];
          cPositions[1] := j;
          break;
        end
      end;
      // Did no history entry match the new values?
      if j = 0 then begin
        if cCount < kMaxHistory then Inc(cCount);
        // Shift all entries down
        for k := cCount downto 2 do cPositions[k] := cPositions[k - 1];
        cPositions[1] := cCount;
        // Add the entry to the history
        sl.Clear;
        sl.Add(number);
        sl.Add(receiver);
        TCommonData.WriteFields(HistorySection, histkey + IntToStr(cCount),
            sl);
      end;
    except
      on EOutOfMemory do begin;
        MessageDlg(GetLabelStr(KIErrOutOfMemory), mtError, [mbOK], 0);
      end;
    end;
    sl.Free;
  end;

  {
    Returns the number of history entries.

    @return history entry count
  }
  class function THistory.Count: integer;
  begin
    Initialize;
    Result := cCount;
  end;

  {
    Gets a number-receiver pair from the history.

    @param index history index for the value
    @param number var param for phone number
    @param receiver var param for receivers name
    @return true, if the entry exist
  }
  class function THistory.Get(const index: Integer;
      var number, receiver: String): Boolean;
  var
    sl: TObservedStringList;
  begin
    Result := false;
    Initialize;
    sl := nil;
    if (index <= cCount) and (index > 0) then begin
      try
        sl := TObservedStringList.Create;
        TCommonData.ReadFields(HistorySection,
            histkey + IntToStr(cPositions[index]), sl);
        if sl.count >= 2 then begin
          number := sl.Strings[0];
          receiver := sl.Strings[1];
          Result := true;
        end;
      except
        on EOutOfMemory do begin;
          MessageDlg(GetLabelStr(KIErrOutOfMemory), mtError, [mbOK], 0);
        end;
      end;
      sl.Free;
    end;
  end;

end.
