{
    Respond, the fax number form daemon
    Copyright (C) 2000 Horst F <horstf@gmx.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}
unit Options;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, Grids, ComCtrls,
  Header, CommonDataAccess;

type
  TOptForm = class(TForm)
    PortLabel: TLabel;
    Port: TEdit;
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Words: TStringGrid;
    LabelsLabel: TLabel;
    DelayedLabel: TLabel;
    DelayedList: TComboBox;
    LoadNetapi: TCheckBox;
    CBSaveHist: TCheckBox;
    CBQueryReceiver: TCheckBox;
    CBQueryUser: TCheckBox;
    CBQuerySender: TCheckBox;
    PCPages: TPageControl;
    TSOptions: TTabSheet;
    TSLabels: TTabSheet;
    CBLookupHost: TCheckBox;
    LBMaxHistory: TLabel;
    MaxHistory: TEdit;
    CBMore: TCheckBox;
    procedure PortKeyPress(Sender: TObject; var Key: Char);
    procedure WordsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure WordsClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private-Deklarationen }
    SavedText : String;
    OKClicked: Boolean;
    procedure WriteValues;
    procedure WriteLabels;
    function BoolToInt(value: Boolean): Integer;
  public
    { Public-Deklarationen }
  end;

var
  OptForm: TOptForm;

implementation

var
  SavedPort: String[5];
  SavedMaxHistory: String[2];
  SavedDelayed: Integer;
  SavedLookupHost, SavedMore,
  SavedLoadNetAPI, SavedSaveHist, SavedQueryReceiver,
  SavedQueryUser, SavedQuerySender: Boolean;
  SavedLabels: Array[KeyIndexes]of String;

{$R *.DFM}

function UnColon(const s: String): String;
{ Remove ':' at the end of a string }                     
begin
  if s[Length(s)] = ':' then
    Result := Copy(s, 1, Length(s)-1)
  else Result := s;
end;

procedure TOptForm.PortKeyPress(Sender: TObject; var Key: Char);
{ Let user only type digits for the port }
begin
  if ( Key >= #30 ) and ( Key <= #255 ) then
   if ( Key < '0' ) or ( Key > '9' ) then Key := #0;
end;

procedure TOptForm.WordsKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
{ Key processing for label list }
begin
  if Shift = [] then with Words do case Key of
  VK_ESCAPE: Cells[1,Row] := SavedText;     { Restore old value }
  VK_HOME, VK_END, VK_LEFT, VK_RIGHT: begin
      EditorMode := True;
    end;
  end;
end;

procedure TOptForm.WordsClick(Sender: TObject);
{ Save value on enter a cell to allow restoring with ESC }
begin
  SavedText := Words.Cells[1,Words.Row];
end;

procedure TOptForm.FormCreate(Sender: TObject);
var
  s: String;
  j: KeyIndexes;
begin
  ProcessOptions();
  TCommonData.BindTableToIniFile(OptionsSection, OptionsFile);
  TCommonData.BindTableToIniFile(LabelsSection, LabelFile);
  { Set labels from ini file }
  OKBtn.Caption := GetLabelStr(KIOK);
  CancelBtn.Caption := GetLabelStr(KICancel);
  PortLabel.Caption := GetLabelStr(KIPort);
  DelayedLabel.Caption := GetLabelStr(KIDelayed);
  LoadNetapi.Caption := GetLabelStr(KILoadNetAPI);
  CBSaveHist.Caption := GetLabelStr(KISAveHist);
  LabelsLabel.Caption := GetLabelStr(KILabels);
  CBQueryReceiver.Caption :=
    GetLabelStr(KIReceiver)+' - '+UnColon(GetLabelStr(KIRecvName));
  CBQueryUser.Caption :=
    GetLabelStr(KISender)+' - '+UnColon(GetLabelStr(KIUser));
  CBQuerySender.Caption :=
    GetLabelStr(KISender)+' - '+UnColon(GetLabelStr(KISendName));
  LBMaxHistory.Caption := GetLabelStr(KIMaxHistory);
  CBLookupHost.Caption := GetLabelStr(KILookupHost);
  CBMore.Caption := GetLabelStr(KIMoreEnabled);
{  Caption := CProgName+' '+UnShortCut(GetLabelStr(KIOptions));}
  { Possibilities for delayed sending }
  DelayedList.Clear;
  for j := KIdOff to KIdEdit do
    DelayedList.Items.Add( GetLabelStr(j) );
  DelayedList.Itemindex :=
      Abs(TCommonData.ReadInteger(OptionsSection, DelayedKey, 0)) mod 3;
  { Label list }
  Words.RowCount := KeyCount;
  for j := KIFirst to KILast do begin
    s := GetLabelStr(j);
    Words.Cells[0,Ord(j)] := CKeys[j];
    Words.Cells[1,Ord(j)] := s;
  end;
  { Other options }
  Port.Text :=
      IntToStr(TCommonData.ReadInteger(OptionsSection, PortValueKey,
      DefaultPort));
  LoadNetapi.Checked :=
      TCommonData.ReadInteger(OptionsSection, LoadNetAPIKey, 0) <> 0;
  CBQueryReceiver.Checked :=
      TCommonData.ReadInteger(OptionsSection, QueryReceiverNameKey, 1) <> 0;
  CBQueryUser.Checked :=
      TCommonData.ReadInteger(OptionsSection, QueryUserKey, 1) <> 0;
  CBQuerySender.Checked :=
      TCommonData.ReadInteger(OptionsSection, QuerySenderNameKey, 1) <> 0;
  CBSaveHist.Checked :=
      TCommonData.ReadInteger(OptionsSection, SaveHistoryKey, 0) <> 0;
  MaxHistory.Text :=
      IntToStr(TCommonData.ReadInteger(OptionsSection, MaxHistKey, 10));
  CBLookupHost.Checked :=
      TCommonData.ReadInteger(OptionsSection, LookupHostKey, 0) <> 0;
  CBMore.Checked :=
      TCommonData.ReadInteger(OptionsSection, MoreKey, 0) <> 0;
  { Save current values for dirty check }
  SavedPort := Port.Text;
  SavedDelayed := DelayedList.ItemIndex;
  SavedLoadNetAPI := LoadNetapi.Checked;
  SavedQueryReceiver := CBQueryReceiver.Checked;
  SavedQueryUser := CBQueryUser.Checked;
  SavedQuerySender :=CBQuerySender.Checked;
  SavedSaveHist := CBSaveHist.Checked;
  SavedMaxHistory := MaxHistory.Text;
  SavedLookupHost := CBLookupHost.Checked;
  SavedMore := CBMore.Checked;
  for j := KIFirst to KILast do begin
    SavedLabels[j] := Words.Cells[1,Ord(j)];
  end;
  OKClicked := False;
end;

function TOptForm.BoolToInt(value: Boolean): Integer;
begin
  if value then Result := 1
  else Result := 0;
end;

procedure TOptForm.WriteValues;
{ Write option values to ini file }
begin
  { Wenn OK gedrckt, nderungen in WIN.INI speichern: }
  TCommonData.WriteString(OptionsSection, PortValueKey, Port.Text);
  TCommonData.WriteInteger(OptionsSection, DelayedKey, DelayedList.ItemIndex);
  TCommonData.WriteInteger(OptionsSection, LoadNetAPIKey,
      BoolToInt(LoadNetapi.Checked));
  TCommonData.WriteInteger(OptionsSection, QueryReceiverNameKey,
      BoolToInt(CBQueryReceiver.Checked));
  TCommonData.WriteInteger(OptionsSection, QueryUserKey,
      BoolToInt(CBQueryUser.Checked));
  TCommonData.WriteInteger(OptionsSection, QuerySenderNameKey,
      BoolToInt(CBQuerySender.Checked));
  TCommonData.WriteInteger(OptionsSection, SaveHistoryKey,
      BoolToInt(CBSaveHist.Checked));
  TCommonData.WriteInteger(OptionsSection, LookupHostKey,
      BoolToInt(CBLookupHost.Checked));
  TCommonData.WriteInteger(OptionsSection, MoreKey,
      BoolToInt(CBMore.Checked));
  { Flush }
  TCommonData.Flush(OptionsSection);
end;

procedure TOptForm.WriteLabels;
{ Write label values to ini file }
var
  j: KeyIndexes;
  p: Array[0..255]of Char;
begin
  for j := KIFirst to KILast do begin
    StrPCopy( @p, Words.Cells[1,Ord(j)]);
    if Words.Cells[1,Ord(j)] <> CLabelValues[j] then
        TCommonData.WriteString(LabelsSection, CKeys[j], Words.Cells[1,Ord(j)])
    else
        { Remove keys for default values }
        TCommonData.DeleteKey(LabelsSection, CKeys[j]);
  end;
  { Flush label ini file }
  TCommonData.Flush(LabelsSection);
end;

procedure TOptForm.OKBtnClick(Sender: TObject);
begin
  OKClicked := True;
  Close;
end;

procedure TOptForm.CancelBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TOptForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  vdirty, ldirty: Boolean;
  j: KeyIndexes;
  i: Word;
  p: Array[0..255]of Char;
begin
  CanClose := True;
  vdirty := False;
  ldirty := False;
  { Check for option changes }
  if (SavedPort <> Port.Text) or
     (SavedDelayed <> DelayedList.ItemIndex) or
     (SavedLoadNetAPI <> LoadNetapi.Checked) or
     (SavedQueryReceiver <> CBQueryReceiver.Checked) or
     (SavedQueryUser <> CBQueryUser.Checked) or
     (SavedQuerySender <> CBQuerySender.Checked) or
     (SavedSaveHist <> CBSaveHist.Checked) or
     (SavedLookupHost <> CBLookupHost.Checked) or
     (SavedMore <> CBMore.Checked)
    then vdirty := True;
  { Check for label changes }
  for j := KIFirst to KILast do
    if SavedLabels[j] <> Words.Cells[1,Ord(j)] then begin
      ldirty := True;
      break;
    end;
  i := IDNO;
  if (not OKClicked) and (vdirty or ldirty) then begin
    GetLabelPChar(KIQuerySave, @p, 255);
    i := Application.MessageBox(@p, 'Config', MB_YESNOCANCEL
                                           or MB_ICONQUESTION
{$IFDEF WIN32}
                                           or MB_RTLREADING
{$ENDIF}
                                              );
  end;
  if i = IDCANCEL then CanClose := False
  else if (OKClicked) or (i = IDYES) then begin
    if vdirty then WriteValues;
    if ldirty then WriteLabels;
  end;
end;

procedure TOptForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  TCommonData.ReleaseTable(OptionsSection);
end;

end.
